#ifndef _comm_Management_hpp
#define _comm_Management_hpp

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <qpid/client/FailoverManager.h>
#include <qpid/client/Session.h>
#include <qpid/client/AsyncSession.h>
#include <qpid/client/Message.h>
#include <qpid/client/MessageReplayTracker.h>

#include "demo_common.h"

using namespace qpid;
using namespace qpid::client;
using namespace qpid::framing;

using namespace std;

namespace demo {

class asynCommManagement : public FailoverManager::ReconnectionStrategy
{
   public: 
      asynCommManagement(int nodeInd, string& hostIP, int nodes, int port);
      ~asynCommManagement(); 
      void editUrlList(std::vector<Url>& urls);
      void reconnect(); 
      void initConnect(); 
      void resetSession(); 
      AsyncSession& curSession();                   //Session& curSession(); 
      MessageReplayTracker& getTracker(); 
   private: 
      ConnectionSettings cs;
      FailoverManager  *fmp;
      Connection     curConn;
      AsyncSession   curSess;                       //Session   curSess;
      MessageReplayTracker myTracker;

      vector<Url>  urls;
      Url  lastUrl; 
      bool isReconnecting; 
};

asynCommManagement::asynCommManagement(int nodeInd, string& hostIP, int nodes, int port): myTracker(100)
{
   in_addr_t net, host;
   struct in_addr startip, theip;   // ip address structure
   int adjust; 
   
   /* nodeInd let us know the first node we try our connection to  */
   adjust = nodeInd % nodes; 
   inet_aton(hostIP.c_str(), &startip);
   host = inet_lnaof(startip);
   net = inet_netof(startip);

   for (int i=0; i< nodes; i++) {
       ostringstream os; 

       theip = inet_makeaddr(net, host+ (i+adjust) % nodes);
       os << "amqp:tcp:" << inet_ntoa(theip) << ":"  << port;

       cout << "debugging: "  << os.str() << endl; 
       
       Url myurl(os.str());
       
       urls.push_back(myurl);

       if ( i == 0 )   {
           cs.host = inet_ntoa(theip);
           cs.port = port;
           cs.heartbeat = HB_INTERVAL;
           lastUrl = myurl; 
           cout << "debugging: " << "starting from " << os.str() << endl; 
       }
   }

   fmp = new FailoverManager(cs, this); 
   isReconnecting = false; 
}

asynCommManagement::~asynCommManagement()
{
    delete fmp; 
}

void asynCommManagement::editUrlList(std::vector<Url>& urls)
{
     if ( ! isReconnecting ) 
          return; 

     if  (  urls.size() > 1 ) {

         if ( urls.back()  != lastUrl ) {
              std::vector<Url>::iterator pos ;
              pos = find( urls.begin(), urls.end(), lastUrl); 
              if (pos != urls.end()) {
                  urls.erase(pos);
                  urls.push_back(lastUrl);
                  lastUrl = urls.front(); 
              } 
         }
     }   
}

void asynCommManagement::reconnect()
{
   isReconnecting = true; 
   curConn = fmp->connect(urls);
   curSess = curConn.newSession(); 
   myTracker.init(curSess);  // is this necessary?  ->replay() actually reset the session this 
   isReconnecting = false; 
}

void asynCommManagement::initConnect()
{
   curConn = fmp->connect(urls);
   curSess = curConn.newSession(); 
   myTracker.init(curSess); 
}

void asynCommManagement::resetSession()
{
   curSess = curConn.newSession(); 
}

AsyncSession & asynCommManagement::curSession()
{
   return (curSess); 
}

MessageReplayTracker & asynCommManagement::getTracker()
{
  return (myTracker); 
}


}  // end of namespace "demo"


#endif

