/*
    By   Qianfeng Zhang ( frzhang@redhat.com )  November 2009 
*/

#include <qpid/client/FailoverManager.h>
#include <qpid/client/Session.h>
#include <qpid/client/Message.h>
#include <qpid/client/SubscriptionManager.h>

#include <iostream>
#include <sstream> 

#include "FailoverWorker.h"
#include "demo_common.h"

#include <unistd.h>
#include <stdlib.h>

using namespace qpid;
using namespace qpid::client;
using namespace qpid::framing;

using namespace std;
using namespace demo;

static void set_thread_fifo();

namespace demo {

class DataStats  
{
  public:
    DataStats(void *priv); 
    ~DataStats() {} ;
    void run();         //needed by the DataStats_runner
    void sigHandle();   // this is used by the boost.signal

  private:
    struct stats_struct *statp;
    unsigned long  lastVal;
    unsigned long  curVal; 
};

DataStats::DataStats(void *priv)
{
   statp = (struct stats_struct *) priv; 
   statp->orderClosed = 0; 
   lastVal = curVal = 0; 
}

void DataStats::run()
{
   long int val; 

   set_thread_fifo(); 
   while ( system_running ) {
       sleep(1); 
       {
         boost::mutex::scoped_lock  l1(statp->lock); 
         curVal = statp->orderClosed; 
       }
       val = labs(curVal-lastVal);  
       cout << val << " orders  closed  in 1 second " << endl; 
       lastVal = curVal;  
   };
}

void DataStats::sigHandle()
{

}

class DataStats_runner
{
  private:
   DataStats *clsp;
  public:
   DataStats_runner(DataStats& lstner) : clsp(&lstner) {} ;
   ~DataStats_runner() {} ;
   void operator()() {
       clsp->run();
   };
};

class DataStats_sigHandler
{
  private:
   DataStats *clsp;
  public:
   DataStats_sigHandler(DataStats& lstner) : clsp(&lstner) {} ;
   ~DataStats_sigHandler() {} ;
   void operator()() {
       clsp->sigHandle();
   };
};


}  // namespace "demo"

static void set_thread_fifo()
{
   struct sched_param sparm;
   int max_priority;

   max_priority = sched_get_priority_max(SCHED_FIFO);
   sparm.sched_priority = max_priority - 10   ;
   pthread_setschedparam(pthread_self(), SCHED_FIFO, &sparm);
}


