#ifndef _comm_Management_hpp
#define _comm_Management_hpp

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "demo_common.h"

using namespace std;

namespace demo {

class syncCommManagement : public FailoverManager::ReconnectionStrategy
{
   public: 
      syncCommManagement(int nodeInd, string& hostIP, int nodes, int port);
      ~syncCommManagement(); 
      void editUrlList(std::vector<Url>& urls);
      void reconnect(); 
      void initConnect(); 
      Session& curSession(); 
      void resetSession(); 
   private: 
      ConnectionSettings cs;
      FailoverManager  *fmp;
      Connection     curConn;
      Session   curSess;

      vector<Url>  urls;
      Url  lastUrl; 
      bool isReconnecting; 
};

syncCommManagement::syncCommManagement(int nodeInd, string& hostIP, int nodes, int port)
{
   in_addr_t net, host;
   struct in_addr startip, theip;   // ip address structure
   int adjust; 
   
   /* nodeInd let us know the first node we try our connection to  */
   adjust = nodeInd % nodes; 
   inet_aton(hostIP.c_str(), &startip);
   host = inet_lnaof(startip);
   net = inet_netof(startip);

   for (int i=0; i< nodes; i++) {
       ostringstream os; 

       theip = inet_makeaddr(net, host+ (i+adjust) % nodes);
       os << "amqp:tcp:" << inet_ntoa(theip) << ":"  << port;

       cout << "debugging: "  << os.str() << endl; 
       
       Url myurl(os.str());
       
       urls.push_back(myurl);

       if ( i == 0 )   {
           cs.host = inet_ntoa(theip);
           cs.port = port;
           cs.heartbeat = HB_INTERVAL;
           lastUrl = myurl; 
       }
   }

   fmp = new FailoverManager(cs, this); 
   isReconnecting = false; 
}

syncCommManagement::~syncCommManagement()
{
    delete fmp; 
}

void syncCommManagement::editUrlList(std::vector<Url>& urls)
{
     if  ( ! isReconnecting ) 
         return; 
     if  (  urls.size() > 1 ) {

         if ( urls.back()  != lastUrl ) {
              std::vector<Url>::iterator pos ;
              pos = find( urls.begin(), urls.end(), lastUrl); 
              if (pos != urls.end()) {
                  urls.erase(pos);
                  urls.push_back(lastUrl);
                  lastUrl = urls.front(); 
              } 
         }
     }   
}

void syncCommManagement::reconnect()
{
   isReconnecting = true; 
   curConn = fmp->connect(urls);
   curSess = curConn.newSession(); 
   isReconnecting = false; 
}

void syncCommManagement::initConnect()
{
   curConn = fmp->connect(urls);
   curSess = curConn.newSession(); 
}

void syncCommManagement::resetSession()
{
  curSess = curConn.newSession(); 
}


Session & syncCommManagement::curSession()
{
   return (curSess); 
}


}  // end of namespace "demo"


#endif

